#!/bin/sh

# NVIDIA Graphics Driver bug reporting shell script.  This shell
# script will generate a log file named "nvidia-<hostname>-bug-report.log.gz", which
# should be attached when emailing bug reports to NVIDIA.

PATH="/sbin:/usr/sbin:/usr/sfw/bin:/usr/ccs/bin:/usr/openwin/bin:$PATH"

BASE_LOG_FILENAME="nvidia-`hostname`-bug-report.log"

# check if gzip is present
if [ -x  /usr/bin/gzip ] || [ -x /usr/sbin/gzip ] || [ -x /sbin/gzip ] || [ -x /usr/sfw/bin/gzip ] || [ -x /usr/ccs/bin/gzip ]; then
    GZIP_CMD="gzip -c"
    LOG_FILENAME="$BASE_LOG_FILENAME.gz"
    OLD_LOG_FILENAME="$BASE_LOG_FILENAME.old.gz"
else
    GZIP_CMD="cat"
    LOG_FILENAME=$BASE_LOG_FILENAME
    OLD_LOG_FILENAME="$BASE_LOG_FILENAME.old"
fi


NVIDIA_BUG_REPORT_CHANGE="$Change: 10129064 $"
NVIDIA_BUG_REPORT_VERSION=`echo $NVIDIA_BUG_REPORT_CHANGE | tr -c -d [0-9]`

#
# echo_metadata() - echo metadata of specified file
#

echo_metadata() {
    printf "*** ls: "
    /bin/ls -E "$1" 2> /dev/null

    if [ $? -ne 0 ]; then
        # Run dumb ls -l. We might not get one-second mtime granularity, but
        # that is probably okay.
        ls -l "$1"
    fi
}

#
# append() - append the contents of the specified file to the log
#

append() {
    (
        echo "____________________________________________"
        echo ""

        if [ ! -f "$1" ]; then
            echo "*** $1 does not exist"
        elif [ ! -r "$1" ]; then
            echo "*** $1 is not readable"
        else
            echo "*** $1"
            echo_metadata "$1"
            cat  "$1"
        fi
        echo ""
    ) | $GZIP_CMD >> $LOG_FILENAME
}

#
# append_silent() - same as append(), but don't print anything
# if the file does not exist
#

append_silent() {
    (
        if [ -f "$1" -a -r "$1" ]; then
            echo "____________________________________________"
            echo ""
            echo "*** $1"
            echo_metadata "$1"
            cat  "$1"
            echo ""
        fi
    ) | $GZIP_CMD >> $LOG_FILENAME
}

#
# append_glob() - use the shell to expand a list of files, and invoke
# append() for each of them
#

append_glob() {
    for i in `ls $1 2> /dev/null;`; do
        append "$i"
    done
}



#
# Start of script
#

# move any old log file (zipped) out of the way

if [ -f $LOG_FILENAME ]; then
    mv $LOG_FILENAME $OLD_LOG_FILENAME
fi


# make sure what we can write to the log file

touch $LOG_FILENAME 2> /dev/null

if [ "$?" -ne "0" ]; then
    echo
    echo "ERROR: Working directory is not writable; please cd to a directory"
    echo "       where you have write permission so that the $LOG_FILENAME"
    echo "       file can be written."
    echo
    exit 1
fi


# print a start message to stdout

echo ""
echo "Running `basename $0`...";


# print prologue to the log file

(
    echo "____________________________________________"
    echo ""
    echo "Start of NVIDIA system configuration collection script."
    echo ""
    echo "nvidia-bug-report.sh Version: $NVIDIA_BUG_REPORT_VERSION"
    echo ""
    echo "Date: `date`"
    echo "uname: `uname -a`"
    echo ""
) | $GZIP_CMD >> $LOG_FILENAME


# append useful info

(
    echo ""
    echo "____________________________________________"
    echo ""
    echo "NVIDIA KERNEL MODULE:"
    ( modinfo | grep nvidia | cut -d'(' -f2 | cut -d')' -f1 ) 2> /dev/null
) | $GZIP_CMD >> $LOG_FILENAME

(
    echo ""
    echo "____________________________________________"
    echo ""
    echo "NVIDIA devices: "
    ( prtconf -vp | grep -i '10de,' | sed 's/^ [ ]*//' ) 2> /dev/null
) | $GZIP_CMD >> $LOG_FILENAME

(
    echo ""
    echo "____________________________________________"
    echo ""
    echo "console device: "
    prtconf -F 2> /dev/null
) | $GZIP_CMD >> $LOG_FILENAME

(
    echo ""
    echo "____________________________________________"
    echo ""
    echo "CPU Information: "
    psrinfo -vp 2> /dev/null
    psrinfo -v  2> /dev/null
) | $GZIP_CMD >> $LOG_FILENAME

(
    echo ""
    echo "____________________________________________"
    echo ""
    echo "Loaded modules: "
    modinfo 2> /dev/null
) | $GZIP_CMD >> $LOG_FILENAME

(
    echo ""
    echo "____________________________________________"
    echo ""
    echo "EEPROM: "
    eeprom 2> /dev/null
) | $GZIP_CMD >> $LOG_FILENAME

append "/etc/release"

(
    echo ""
    echo "____________________________________________"
    echo ""
    echo "sysdef: "
    sysdef 2> /dev/null
) | $GZIP_CMD >> $LOG_FILENAME

(
    echo ""
    echo "____________________________________________"
    echo ""
    echo "isainfo -kv: "
    isainfo -kv 2> /dev/null
) | $GZIP_CMD >> $LOG_FILENAME

(
    echo ""
    echo "____________________________________________"
    echo ""
    echo "pkginfo -l NVDAgraphics"
    pkginfo -l NVDAgraphics 2> /dev/null
) | $GZIP_CMD >> $LOG_FILENAME

(
    echo ""
    echo "____________________________________________"
    echo ""
    echo "pkgchk NVDAgraphics"
    pkgchk NVDAgraphics 2> /dev/null
) | $GZIP_CMD >> $LOG_FILENAME

(
    echo ""
    echo "____________________________________________"
    echo ""
    echo "ls -l /dev/fb* /dev/nvidiactl"
    ls -l /dev/fb* /dev/nvidiactl 2> /dev/null
) | $GZIP_CMD >> $LOG_FILENAME

(
    echo ""
    echo "____________________________________________"
    echo ""
    echo "prtconf -v /dev/nvidia*"
    for i in /dev/nvidia*; do
        prtconf -v $i 2> /dev/null
    done
) | $GZIP_CMD >> $LOG_FILENAME

(
    echo ""
    echo "____________________________________________"
    echo ""
    echo "prtconf -vp"
    prtconf -vp 2> /dev/null
) | $GZIP_CMD >> $LOG_FILENAME

# append the X log; also, extract the config file named in the X log
# and append it; look for X log files with names of the form:
# /var/log/{XFree86,Xorg}.{0,1,2,3,4,5,6,7}.{log,log.old}

xconfig_file_list=

for log_basename in /var/log/XFree86 /var/log/Xorg; do
    for i in 0 1 2 3 4 5 6 7; do
        for log_suffix in log log.old; do
            log_filename="${log_basename}.${i}.${log_suffix}"
            append_silent "${log_filename}"

            # look for the X configuration file referenced by this X log
            if [ -f ${log_filename} -a -r ${log_filename} ]; then
                j=`grep "Using config file" ${log_filename} | cut -f 2 -d \"`
                if [ "$j" ]; then
                    # multiple of the logs we find above might reference the
                    # same X configuration file; keep a list of which X
                    # configuration files we find, and only append X
                    # configuration files we have not already appended
                    echo "${xconfig_file_list}" | grep ":${j}:" > /dev/null
                    if [ "$?" != "0" ]; then
                        xconfig_file_list="${xconfig_file_list}:${j}:"
                        append "$j"
                    fi
                fi
            fi
        done
    done
done


# get low-level system information from SMBIOS

(
    echo ""
    echo "____________________________________________"
    echo ""
    echo "SMBIOS:"
    smbios 2> /dev/null
) | $GZIP_CMD >> $LOG_FILENAME


# get any relevant kernel messages

(
    echo "____________________________________________"
    echo ""
    echo "Scanning kernel log files for NVRM messages:"
    echo ""

    for i in /var/adm/messages /var/adm/messages.? ; do
        if [ -f $i -a -r $i ]; then
            echo "  $i:"
            ( cat $i | grep NVRM ) 2> /dev/null
        else
            echo "$i is not readable"
        fi
    done
) | $GZIP_CMD >> $LOG_FILENAME

# append dmesg output

(
    echo ""
    echo "____________________________________________"
    echo ""
    echo "dmesg:"
    dmesg 2> /dev/null
) | $GZIP_CMD >> $LOG_FILENAME

# In case of failure, if xset returns with delay, we print the
# message from check "$?" & if it returns error immediately before kill,
# we directly write the error to the log file.

(
    echo "____________________________________________"
    echo ""
    echo "xset -q:"

    xset -q 2>&1 & sleep 1 ; kill -9 $! > /dev/null 2>&1

    if [ $? -eq 0 ]; then
        # The xset process is still there,
        echo "xset could not connect to an X server"
    fi
) | $GZIP_CMD >> $LOG_FILENAME

# In case of failure, if nvidia-settings returns with delay, we print the
# message from check "$?" & if it returns error immediately before kill,
# we directly write the error to the log file.

(
    echo "____________________________________________"
    echo ""
    echo "nvidia-settings -q all:"

    DPY="$DISPLAY"
    [ "$DPY" ] || DPY=":0"
    DISPLAY= nvidia-settings -c "$DPY" -q all 2>&1 & sleep 1 ; kill -9 $! > /dev/null 2>&1

    if [ $? -eq 0 ]; then
        # The nvidia-settings process is still there.
        echo "nvidia-settings could not connect to an X server"
    fi
) | $GZIP_CMD >> $LOG_FILENAME

(
    echo "____________________________________________"

    # print epilogue to log file

    echo ""
    echo "End of NVIDIA bug report log file."
) | $GZIP_CMD >> $LOG_FILENAME


# Done

echo " complete."
echo ""
echo "The file $LOG_FILENAME has been created"
echo ""
